use macroquad::audio::{load_sound_from_bytes, set_sound_volume, Sound};
use macroquad::color::Color;
use macroquad::miniquad::FilterMode;
use macroquad::prelude::{Image, ImageFormat};
use macroquad::text::{
    camera_font_scale, draw_text_ex, load_ttf_font_from_bytes, Font, TextParams,
};
use macroquad::texture::Texture2D;

pub struct Assets {
    pub blobcat_texture: Texture2D,
    pub cloud_texture: Texture2D,
    pub sun_texture: Texture2D,
    pub grass_top_texture: Texture2D,
    pub grass_bottom_texture: Texture2D,
    pub road_texture: Texture2D,
    pub house_texture: Texture2D,
    pub bike_texture: Texture2D,
    pub projectiles_texture: Texture2D,
    pub particles_texture: Texture2D,
    pub menu_texture: Texture2D,
    pub controls_texture: Texture2D,
    pub miss_sound: Sound,
    pub shoot_sound: Sound,
    pub hit_sound: Sound,
    pub lose_life_sound: Sound,
    pub font: Font,
}

impl Assets {
    pub async fn load() -> Self {
        let blobcat_basic = Image::from_file_with_format(
            include_bytes!("../assets/blobcat_basic.png"),
            Some(ImageFormat::Png),
        );
        let blobcat_texture = Texture2D::from_image(&blobcat_basic);
        blobcat_texture.set_filter(FilterMode::Nearest);

        let cloud = Image::from_file_with_format(
            include_bytes!("../assets/cloud1.png"),
            Some(ImageFormat::Png),
        );
        let cloud_texture = Texture2D::from_image(&cloud);
        cloud_texture.set_filter(FilterMode::Nearest);

        let sun = Image::from_file_with_format(
            include_bytes!("../assets/sun.png"),
            Some(ImageFormat::Png),
        );
        let sun_texture = Texture2D::from_image(&sun);
        sun_texture.set_filter(FilterMode::Nearest);

        let grass_top1 = Image::from_file_with_format(
            include_bytes!("../assets/grass_top.png"),
            Some(ImageFormat::Png),
        );
        let grass_top_texture = Texture2D::from_image(&grass_top1);
        grass_top_texture.set_filter(FilterMode::Nearest);

        let grass_bottom1 = Image::from_file_with_format(
            include_bytes!("../assets/grass_bottom.png"),
            Some(ImageFormat::Png),
        );
        let grass_bottom_texture = Texture2D::from_image(&grass_bottom1);
        grass_bottom_texture.set_filter(FilterMode::Nearest);

        let road = Image::from_file_with_format(
            include_bytes!("../assets/road.png"),
            Some(ImageFormat::Png),
        );
        let road_texture = Texture2D::from_image(&road);
        road_texture.set_filter(FilterMode::Nearest);

        let house = Image::from_file_with_format(
            include_bytes!("../assets/house.png"),
            Some(ImageFormat::Png),
        );
        let house_texture = Texture2D::from_image(&house);
        house_texture.set_filter(FilterMode::Nearest);

        let bike = Image::from_file_with_format(
            include_bytes!("../assets/bike.png"),
            Some(ImageFormat::Png),
        );
        let bike_texture = Texture2D::from_image(&bike);
        bike_texture.set_filter(FilterMode::Nearest);

        let projectiles = Image::from_file_with_format(
            include_bytes!("../assets/projectiles.png"),
            Some(ImageFormat::Png),
        );
        let projectiles_texture = Texture2D::from_image(&projectiles);
        projectiles_texture.set_filter(FilterMode::Nearest);

        let particles = Image::from_file_with_format(
            include_bytes!("../assets/particles.png"),
            Some(ImageFormat::Png),
        );
        let particles_texture = Texture2D::from_image(&particles);
        particles_texture.set_filter(FilterMode::Nearest);

        let menu = Image::from_file_with_format(
            include_bytes!("../assets/logo.png"),
            Some(ImageFormat::Png),
        );
        let menu_texture = Texture2D::from_image(&menu);
        menu_texture.set_filter(FilterMode::Nearest);

        let controls = Image::from_file_with_format(
            include_bytes!("../assets/controls.png"),
            Some(ImageFormat::Png),
        );
        let controls_texture = Texture2D::from_image(&controls);
        controls_texture.set_filter(FilterMode::Nearest);

        let font = load_ttf_font_from_bytes(include_bytes!("../assets/plutostardust.ttf")).unwrap();

        let miss_sound = load_sound_from_bytes(include_bytes!("../assets/miss.ogg"))
            .await
            .unwrap();

        let shoot_sound = load_sound_from_bytes(include_bytes!("../assets/shoot.ogg"))
            .await
            .unwrap();

        let hit_sound = load_sound_from_bytes(include_bytes!("../assets/hit.ogg"))
            .await
            .unwrap();

        let lose_life_sound = load_sound_from_bytes(include_bytes!("../assets/lose-life.ogg"))
            .await
            .unwrap();

        Self {
            blobcat_texture,
            cloud_texture,
            sun_texture,
            grass_top_texture,
            grass_bottom_texture,
            road_texture,
            house_texture,
            bike_texture,
            projectiles_texture,
            particles_texture,
            menu_texture,
            controls_texture,
            miss_sound,
            shoot_sound,
            hit_sound,
            lose_life_sound,
            font,
        }
    }
}

pub fn draw_string(text: &str, x: f32, y: f32, color: Color, font: Font) {
    let (f_size, f_scale, f_aspect) = camera_font_scale(8.0);
    draw_text_ex(
        text,
        x,
        y,
        TextParams {
            font_scale: f_scale,
            font_size: f_size,
            font_scale_aspect: f_aspect,
            font,
            color,
            ..Default::default()
        },
    )
}
